/*
工具: 编码\加密\解密
*/

package tools

import (
	"bytes"
	"crypto/aes"
	"crypto/cipher"
	"crypto/md5"
	"encoding/base64"
	"fmt"
	"io"
)

// 对称加密解密 ----------------------------------------------------

func PKCS7Padding(ciphertext []byte, blockSize int) []byte {
	padding := blockSize - len(ciphertext)%blockSize
	padtext := bytes.Repeat([]byte{byte(padding)}, padding)
	return append(ciphertext, padtext...)
}

func PKCS7UnPadding(origData []byte) []byte {
	length := len(origData)
	unpadding := int(origData[length-1])
	return origData[:(length - unpadding)]
}

// AES加密
func AesEncrypt(origData, key []byte) ([]byte, error) {
	block, err := aes.NewCipher(key)
	if err != nil {
		return nil, err
	}
	blockSize := block.BlockSize()
	origData = PKCS7Padding(origData, blockSize)
	blockMode := cipher.NewCBCEncrypter(block, key[:blockSize])
	crypted := make([]byte, len(origData))
	blockMode.CryptBlocks(crypted, origData)
	return crypted, nil
}

// AES解密
func AesDecrypt(crypted, key []byte) ([]byte, error) {
	block, err := aes.NewCipher(key)
	if err != nil {
		return nil, err
	}
	blockSize := block.BlockSize()
	blockMode := cipher.NewCBCDecrypter(block, key[:blockSize])
	origData := make([]byte, len(crypted))
	blockMode.CryptBlocks(origData, crypted)
	origData = PKCS7UnPadding(origData)
	return origData, nil
}

// 解密授权文件  encrypt 加密吗?
func License(str string, key string, encrypt bool) string {
	AesKey := []byte(key) //秘钥长度为16的倍数
	if encrypt {          // 加密
		encrypted, err := AesEncrypt([]byte(str), AesKey)
		if err != nil {
			return ""
		}
		return base64.StdEncoding.EncodeToString(encrypted)
	} else { // 解密
		enStr, err := base64.StdEncoding.DecodeString(str)
		if err != nil {
			return ""
		}
		origin, err := AesDecrypt(enStr, AesKey)
		if err != nil {
			return ""
		}
		return string(origin)
	}
}

// Base64
func Base64Encode(src []byte) []byte {
	return []byte(base64.StdEncoding.EncodeToString(src))
}

// MD5
func MD5(str string) string {
	m := md5.New()
	_, err := io.WriteString(m, str)
	if err != nil {
		return ""
	}
	arr := m.Sum(nil)
	return fmt.Sprintf("%x", arr)
}
